@@ -1,7 +1,8 @@
 `dredge` <-
 function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
-		 fixed = NULL, m.max = NA, m.min = 0, subset, marg.ex = NULL,
-		 trace = FALSE, varying, extra, ct.args = NULL, ...) {
+		 fixed = NULL, m.max = NA, m.min = 0, subset,
+		 trace = FALSE, varying, extra, ct.args = NULL,
+		 ...) {
 
 	gmEnv <- parent.frame()
 	gmCall <- .getCall(global.model)
@@ -10,30 +11,32 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	if (is.null(gmCall)) {
 		gmCall <- substitute(global.model)
 		if(!is.call(gmCall)) {
-			stop("need a 'global.model' with call component. Consider using ", 
+			stop("need a 'global.model' with a call component. Consider using ", 
 				if(inherits(global.model, c("gamm", "gamm4"))) 
 					"'uGamm'" else "'updateable'")
 		}
 		#"For objects without a 'call' component the call to the fitting function \n",
 		#" must be used directly as an argument to 'dredge'.")
-		# NB: this is unlikely to happen:
+		# NB: this is unlikely to happen
 		if(!is.function(eval(gmCall[[1L]], parent.frame())))
-			gettext('could not find function "%s"', deparse(gmCall[[1L]], 
-				control = NULL), domain = "R")
+			.cry(NA, "could not find function '%s'", deparse(gmCall[[1L]],
+				control = NULL))
 	} else {
-		# if 'update' method does not expand dots, we have a problem
-		# with expressions like ..1, ..2 in the call.
-		# So, try to replace them with respective arguments in the original call
+		# if 'update' method does not expand dots, we have a problem with
+		# expressions like ..1, ..2 in the call. So try to replace them with
+		# respective arguments in the original call
 		is.dotted <- grep("^\\.\\.", sapply(as.list(gmCall), deparse))
 		if(length(is.dotted) > 0L) {
 			substGmCall <- substitute(global.model)
 			if(is.name(substGmCall)) {
-				.cry(NA, "call to 'global.model' contains '...' arguments and cannot be updated: %s", deparse(gmCall, control = NULL))
+				.cry(NA, "call to 'global.model' contains unexpanded dots and cannot be updated: \n%s",
+					 deparse(gmCall, control = NULL))
 			} else gmCall[is.dotted] <-
 				substitute(global.model)[names(gmCall[is.dotted])]
 		}
 		
-		## object from 'run.mark.model' has $call of 'make.mark.model' - fixing it here:
+		# object from 'run.mark.model' has $call of 'make.mark.model' - fixing
+		# it here:
 		if(inherits(global.model, "mark") && gmCall[[1L]] == "make.mark.model") {
 			gmCall <- call("run.mark.model", model = gmCall, invisible = TRUE)
 		}
@@ -53,11 +56,29 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	}
 	
 	rankArgs <- list(...)
+
+	if(any(wrongarg <- names(rankArgs) == "marg.ex")) {
+		.cry(NA, "argument \"marg.ex\" is defunct and has been ignored",
+			 warn = TRUE)
+		rankArgs <- rankArgs[!wrongarg]
+	}
+	if(any(names(rankArgs) == "na.action")) {
+		.cry("RTFM", "argument \"na.action\" is really inappropriate here",
+			 warn = FALSE)
+	}
+	
 	IC <- .getRank(rank, rankArgs)
+	
+	if(any(wrongarg <- is.na(match(names(rankArgs),
+		c(names(formals(get("rank", environment(IC))))[-1L], names(formals()))))))
+		.cry(NA, "arguments %s are not formal arguments of 'dredge' or 'rank'",
+			 prettyEnumStr(names(rankArgs[wrongarg])), warn = TRUE)
+	
 	ICName <- as.character(attr(IC, "call")[[1L]])
 	
 	if(length(tryCatch(IC(global.model), error = function(e) {
-		e$call <- do.call(substitute, list(attr(IC, "call"), list(x = as.name("global.model"))))
+		e$call <- do.call(substitute, list(attr(IC, "call"), 
+			list(x = as.name("global.model"))))
 		stop(e)
 	})) != 1L) {
 		.cry(NA, "result of '%s' is not of length 1", deparse(attr(IC,
@@ -74,8 +95,7 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 
 	#XXX: use.ranef <- FALSE
 	#if(use.ranef && inherits(global.model, "mer")) {
-		#allTerms <- c(allTerms, paste("(", attr(allTerms0, "random.terms"), ")",
-			#sep = ""))
+		#allTerms <- c(allTerms, paste0("(", attr(allTerms0, "random.terms"), ")"))
 	#}
 
 
@@ -124,10 +144,20 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 			fixed <- fixed[i]
 		}
 	}
+	
+	deps <- attr(allTerms0, "deps")
+	fixed <- union(fixed, rownames(deps)[rowSums(deps, na.rm = TRUE) == ncol(deps)])
 	fixed <- c(fixed, allTerms[allTerms %in% interceptLabel])
 	n.fixed <- length(fixed)
+	if(n.fixed > 0L) message(sprintf(ngettext(n.fixed, "Fixed term is %s", "Fixed terms are %s"),
+		prettyEnumStr(fixed)))
+
 	termsOrder <- order(allTerms %in% fixed)
 	allTerms <- allTerms[termsOrder]
+	
+	di <- match(allTerms, rownames(deps))
+	deps <- deps[di, di]	
+	
 	gmFormulaEnv <- environment(as.formula(formula(global.model), env = gmEnv))
 	# TODO: gmEnv <- gmFormulaEnv ???
 
@@ -257,37 +287,16 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 			
 			rownames(gloFactorTable) <- allTerms0[!(allTerms0 %in% interceptLabel)]
 	
-			
-			subsetExpr <- .substFunc(subsetExpr, ".", function(x, fac, at, vName) {
-				if(length(x) != 2L) .cry(x, "exactly one argument needed, %d given.", length(x) - 1L)
-				if(length(x[[2L]]) == 2L && x[[2L]][[1L]] == "+") {
-					fun <- "all"
-					sx <- as.character(x[[2L]][[2L]])
-				} else {
-					fun <- "any"
-					sx <- as.character(x[[2L]])
-				}
-				#print(sx)
-				dn <- dimnames(fac)
-				#print(dn)
-				#browser()
-				if(!(sx %in% dn[[2L]])) .cry(x, "unknown variable name '%s'", sx)
-				as.call(c(as.name(fun), call("[", vName, as.call(c(as.name("c"), 
-					match(dn[[1L]][fac[, sx]], at))))))
-			}, gloFactorTable, allTerms, as.name("comb"))
-			
+			subsetExpr <- .substFunc(subsetExpr, ".", .sub_dot, gloFactorTable, 
+				allTerms, as.name("comb"))
+			subsetExpr <- .substFunc(subsetExpr, c("{", "Term"), .sub_Term)
 			subsetExpr <- .subst4Vec(subsetExpr, allTerms, as.name("comb"))
 			
 			
 			if(nvarying) {
 				ssValidNames <- c("cVar", "comb", "*nvar*")
-				subsetExpr <- .substFunc(subsetExpr, "V", function(x, cVar, fn) {
-					if(length(x) > 2L) .cry(x, "discarding extra arguments", warn = TRUE)
-					i <- which(fn == x[[2L]])[1L]
-					if(is.na(i)) .cry(x, "'%s' is not a valid name of 'varying' element",
-									  as.character(x[[2L]]), warn = TRUE)
-					call("[[", cVar, i)
-				}, as.name("cVar"), varying.names)
+				subsetExpr <- .substFunc(subsetExpr, "V", .sub_V, 
+					as.name("cVar"), varying.names)
 				if(!all(all.vars(subsetExpr) %in% ssValidNames))
 					subsetExpr <- .subst4Vec(subsetExpr, varying.names,
 											 as.name("cVar"), fun = "[[")
@@ -332,20 +341,6 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	matchCoefCall <- as.call(c(alist(matchCoef, fit1, all.terms = allTerms,
 		  beta = beta, allCoef = TRUE), ct.args))
 	
-	# TODO: allow for 'marg.ex' per formula in multi-formula models
-	if(missing(marg.ex) || (!is.null(marg.ex) && is.na(marg.ex))) {
-		newArgs <- makeArgs(global.model, allTerms, rep(TRUE, length(allTerms)),
-							argsOptions)
-		formulaList <- if(is.null(attr(newArgs, "formulaList"))) newArgs else
-			attr(newArgs, "formulaList")
-
-		marg.ex <- unique(unlist(lapply(sapply(formulaList, formulaMargChk,
-			simplify = FALSE), attr, "marg.ex")))
-		if(!length(marg.ex)) marg.ex <- NULL else
-			cat("Marginality exceptions:", sQuote(marg.ex), "\n")
-	}
-	###
-
 	retColIdx <- if(nvarying) -n.vars - seq_len(nvarying) else TRUE
 
 	prevJComb <- 0L
@@ -357,8 +352,9 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 			
 			comb <- c(as.logical(intToBits(jComb - 1L)[comb.seq]), comb.sfx)
 			nvar <- sum(comb) - nInts
-				
+							
 			if(nvar > m.max || nvar < m.min ||
+			   !formula_margin_check(comb, deps) ||
 			   switch(hasSubset,
 					FALSE,
 					!all(subset[comb, comb], na.rm = TRUE),
@@ -367,18 +363,13 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 					FALSE
 			   )) {
 				isok <- FALSE
-				next;
+				next
 			}
 			
-						
 			newArgs <- makeArgs(global.model, allTerms[comb], comb, argsOptions)
 			formulaList <- if(is.null(attr(newArgs, "formulaList"))) newArgs else
 				attr(newArgs, "formulaList")
 
-				
-			if(!all(vapply(formulaList, formulaMargChk, logical(1L), marg.ex)))  {
-				isok <- FALSE; next;
-			}
 			if(!is.null(attr(newArgs, "problems"))) {
 				print.warnings(structure(vector(mode = "list",
 					length = length(attr(newArgs, "problems"))),
@@ -394,7 +385,6 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 		clVariant <- cl
 		if (nvarying) {
 			cvi <- variants[(iComb - 1L) %% nvariants + 1L, ]
-
 			if(hasSubset == 4L &&
 				!.evalExprIn(subsetExpr, env = ssEnv, enclos = parent.frame(),
 					comb = comb, `*nvar*` = nvar, cVar = flat.variant.Vvals[cvi]))
@@ -428,7 +418,6 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 					tmp[match(names(extraResult1), names(extraResult))] <- extraResult1
 					extraResult1 <- tmp
 				}
-				#row1 <- c(row1, extraResult1)
 			}
 
 			#mcoef1 <- matchCoef(fit1, all.terms = allTerms, beta = beta,
@@ -438,7 +427,7 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 			ll <- logLik(fit1)
 			nobs1 <- nobs(fit1)
 			if(nobs1 != gmNobs) warning(gettextf(
-				"number of observations in model #%d (%d) differs from that in the global model (%d)",
+				"number of observations in model #%d (%d) differs from that in global model (%d)",
 				iComb, nobs1, gmNobs))
 
 			row1 <- c(mcoef1[allTerms], extraResult1,
@@ -499,7 +488,7 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 
 	if(nvarying) {
 		variant.names <- vapply(flat.variant.Vvals, function(x) if(is.character(x)) x else
-			deparse(x, control = NULL, width.cutoff = 20L)[1L], character(1L))
+			deparse(x, control = NULL, width.cutoff = 20L)[1L], "")
 		vnum <- split(seq_len(sum(vlen)), rep(seq_len(nvarying), vlen))
 		names(vnum) <- varying.names
 		for (i in varying.names) ret[, i] <-
