@@ -80,8 +80,8 @@ function(x, typeR = FALSE) .NotYetImplemented()
 `getQIC.coxph` <- function(x, ...) {
 	warning("QIC for coxph is experimental")
 	naive.var <- x[[ if (is.null(x$naive.var)) "var" else "naive.var" ]]
-	# tr <- sum(diag(solve(naive.var) %*% x$var)) ## TODO: use matmult
-	tr <- sum(matmultdiag(solve(naive.var), x$var)) ## TODO: use matmult
+	# tr <- sum(diag(solve(naive.var) %*% x$var))
+	tr <- sum(matmultdiag(solve(naive.var), x$var))
 	ll <- x$loglik[2L]
 	px <- x$n
 	c(2 * (c(QIC = tr, QICu = px) - ll), n = px)
