@@ -1,14 +1,22 @@
-`DebugPrint` <- function(x) { cat(deparse(substitute(x)), "= \n") ; print(x) }
+`DebugPrint` <- function(x) {
+	if(isTRUE(getOption('mumin.debug'))) {
+		cat("~", deparse(substitute(x)), "=");
+		print(x)	
+	}
+}
+	
 `srcc` <- function() {
 	ret <- eval(expression(source("clipboard", local = TRUE)), .GlobalEnv)
 	return(if(ret$visible) ret$value else invisible(ret$value))
 }
 
 `.cry` <-
-function(Call = NA, Message, ..., warn = FALSE) {
-	if(!is.call(Call)) Call <- sys.call(-1L)
-	if(warn) warning(simpleWarning(gettextf(Message, ..., domain = "R-MuMIn"), Call)) else
-		stop(simpleError(gettextf(Message, ..., domain = "R-MuMIn"), Call))
+function(Call = NA, Message, ..., warn = FALSE, domain = paste0("R-", .packageName)) {
+	if (is.character(Call))
+		Call <- call(Call[1L], sys.call(-1L)[[1L]]) else
+		if(!is.call(Call)) Call <- sys.call(-1L)
+	if(warn) warning(simpleWarning(gettextf(Message, ..., domain = domain), Call)) else
+		stop(simpleError(gettextf(Message, ..., domain = domain), Call))
 }
 
 #if (!exists("getElement", mode = "function", where = "package:base", inherits = FALSE)) {
@@ -46,62 +54,7 @@ function(x) all(vapply(x[-1L], identical, logical(1L), x[[1L]]))
 # Check class for each object in a list
 `linherits` <- function(x, whats) {
 	as.logical(vapply(x, inherits, integer(length(whats)), names(whats),
-		which=TRUE)) == whats
-}
-
-# substitute has(a, !b, ...) for !is.na(a) & is.na(b) ..., in expression
-`.substHas` <- function(e) {
-	if(is.expression(e)) e <- e[[1L]]
-	n <- length(e)
-	if(n == 1L) return(e)
-	if(e[[1L]] != "has") {
-		for(i in 1L:n) e[[i]] <- .substHas(e[[i]])
-		return(e)
-	}
-	res <- NULL
-	for(i in seq.int(2L, n)) {
-		ex <- if(length(e[[i]]) == 2L && e[[i]][[1L]] == "!")
-			call("is.na", e[[i]][[2L]]) else
-			call("!", call("is.na", e[[i]]))
-		res <- if(i == 2L) ex else call("&", res, ex)
-	}
-	res <- call("(", res)
-	return(res)
-}
-
-# substitute function calls in 'e'. 'name' is replaced by 'fun.to'.
-#`.substFun` <- function(e, name, fun.to, ignore.I = TRUE) {
-#	if(is.expression(e)) e <- e[[1L]]
-#	n <- length(e)
-#	if(n == 1L && !is.call(e)) return(e)
-#	if(ignore.I && e[[1L]] == "I") return(e)
-#	if(n != 1L) for(i in 2L:n) e[[i]] <- .substFun(e[[i]], name, fun.to, ignore.I = ignore.I)
-#	if(e[[1L]] == name) e[[1L]] <- as.name(fun.to)
-#	return(e)
-#}
-
-# substitute function calls in 'e'. 'func' must take care of the substitution job.
-`.substFunc` <- function(e, name, func = identity, ...) {
-	if(is.expression(e)) e <- e[[1L]]
-	n <- length(e)
-	if(n == 0L) return(e) else if (n == 1L) {
-		if (!is.call(e)) return(e)
-	} else for(i in 2L:n) e[i] <- list(.substFunc(e[[i]], name, func, ...))
-	if(e[[1L]] == name) e <- func(e, ...)
-	return(e)
-}
-
-# evaluate 'expr' in 'env' after adding variables passed as ...
-.evalExprIn <- function(expr, env, enclos, ...) {
-	list2env(list(...), env)
-	eval(expr, envir = env, enclos = enclos)
-}
-
-# substitute names for varName[1], varName[2], ... in expression
-`.subst4Vec` <- function(expr, names, varName, n = length(names), fun = "[") {
-	eval(call("substitute", expr,
-		env = structure(lapply(seq_len(n), function(i) call(fun, varName, i)), names = names)),
-		envir = NULL)
+		which = TRUE)) == whats
 }
 
 # tries to make a list of element names
@@ -130,16 +83,15 @@ function(x) all(vapply(x[-1L], identical, logical(1L), x[[1L]]))
 # vectorized version of .subset_do (used within subset.model.selection)
 `.subset_vdc` <- function(...) apply(cbind(..., deparse.level = 0L), 1L, .subset_dc)
 
-
 `prettyEnumStr` <- function(x, sep = ", ", sep.last = gettext(" and "), quote = TRUE) {
 	n <- length(x)
 	if(is.function(quote))
 		x <- quote(x) else {
 			if(identical(quote, TRUE)) quote <- '"'
-			if(is.character(quote)) x <- paste(quote, x, quote, sep = "")
+			if(is.character(quote)) x <- paste0(quote, x, quote)
 		}
-	paste(x, if(n > 1L) c(rep(sep, n - 2L), sep.last, "") else NULL,
-		collapse = "", sep = "")
+	paste0(x, if(n > 1L) c(rep(sep, n - 2L), sep.last, "") else NULL,
+		collapse = "")
 }
 
 # `splitList` <- function (x, k) {
@@ -178,11 +130,10 @@ function(x) all(vapply(x[-1L], identical, logical(1L), x[[1L]]))
 		vnames <- names(vars)
 		#if(!all(sapply(Call, is.name))) warning("at least some elements do not have syntactic name")
 		if(is.null(vnames)) {
-			names(vars) <- vapply(Call, deparse, character(1L), control = NULL,
-				nlines = 1L)
+			names(vars) <- vapply(Call, deparse, "", control = NULL, nlines = 1L)
 		} else if (any(vnames == "")) {
 			names(vars) <- ifelse(vnames == "", vapply(Call, deparse,
-				character(1L), control = NULL, nlines = 1L), vnames)
+				"", control = NULL, nlines = 1L), vnames)
 		}
 		get("clusterCall")(cluster, getv, vars)
 		# clusterCall(cluster, getv, vars)
